import Vuex from 'vuex'
import Vue from 'vue'
import { deepClone } from 'ap-util/util/ObjectUtil'
import { formatDate } from 'ap-util/util/DateUtil'

// 获取组件的store对象
const getItemByRef = function(ref, data) {
  if (data != null && data.ref == ref) {
    return data
  }
  if (data != null && data.children != null && data.children.length > 0) {
    for (var i = 0; i < data.children.length; i++) {
      var tempData = getItemByRef(ref, data.children[i])
      if (tempData != null) {
        return tempData
      }
    }
  }
}

// 获取组件的父store对象
const getParentItemByRef = function(ref, data) {
  if (data == null || data.children == null || data.children.length < 1) {
    return null
  }
  for (var i = 0; i < data.children.length; i++) {
    if (data.children[i].ref == ref) {
      return data
    }
    var tempData = getParentItemByRef(ref, data.children[i])
    if (tempData != null) {
      return tempData
    }
  }
}

// 获取item的索引
const getItemIndexByRef = function(ref, data) {
  const parentItem = getParentItemByRef(ref, data)
  for (let i = 0; i < parentItem.children.length; i++) {
    if (parentItem.children[i].ref == ref) {
      return i
    }
  }
}

const getNewRef = function(type) {
  // 三位编号：取store根路径rootContainer的seq
  const seq = apDevStore.state.json.seq
  // 更新seq
  apDevStore.commit('seqIncrease')
  return type + '_' + (Array(3).join('0') + seq).slice(-3)
}

const setNewRef = function(item) {
  if (item == null) {
    return
  }
  item.ref = getNewRef(item.type)
  const children = item.children
  if (children != null && children.length > 0) {
    for (let i = 0; i < children.length; i++) {
      setNewRef(children[i])
    }
  }
}

const apDevStore = new Vuex.Store({
  state: {
    json: {
      ref: 'rootContainer', // 默认父容器
      type: 'containerRow',
      seq: 1, // 组件数量计数
      children: []
    },
    currentComps: {
      ref: '',
      type: 'ap-dev-default-prop'
    },
    history: {
      currentIndex: 0,
      data: []
    },
    tempData: {
      copyItem: {},
      currentDesignPanel: 'api-panel',
      currentModel: 'AP'
    }
  },
  strict: true,
  getters: {
    // 获取组件：根据ref
    getItem: (state) => (ref) => {
      return getItemByRef(ref, state.json)
    },
    // 获取json
    getJson: (state) => {
      return state.json
    }
  },
  mutations: {
    // 历史记录：增加
    historyAdd(state, param) {
      if (state.history.currentIndex > 0) {
        state.history.data.splice(0, state.history.currentIndex)
      }
      param.date = formatDate(new Date(), 'MM-dd hh:mm')
      param.data = deepClone(state.json)
      state.history.data.splice(0, 0, param)
      state.history.currentIndex = 0
    },
    // 序列增加
    seqIncrease(state) {
      state.json.seq++
    },
    // 更新当前选择组件
    updateCurrentItem(state, param) {
      const propType = 'ap-dev-' + param.type + '-prop'
      const newObj = {
        type: propType,
        ref: param.ref
      }
      state.currentComps = newObj
    },
    // 重置当前选择组件
    resetCurrentItem(state) {
      this.commit('updateCurrentItem', {
        type: 'default',
        ref: ''
      })
    },

    // 通用：添加组件store数据：参数 -> 父组件ref、添加位置、添加对象
    addItem(state, param) {
      if (param == null) {
        return
      }
      var item = getItemByRef(param.parentRef, state.json)
      // 指定位置插入新元素
      item.children.splice(param.dataIndex, 0, param.obj)
    },
    // 通用：删除组件
    removeItem(state, param) {
      var parent = getParentItemByRef(param.ref, state.json)
      if (!parent) {
        return
      }
      var children = parent.children
      for (var i = 0; i < children.length; i++) {
        if (children[i].ref == param.ref) {
          children.splice(i, 1)
          return
        }
      }
    },

    // 属性修改：初始化组件options
    propInitItemOptions(state, param) {
      console.log('属性修改：store -> ' + param.ref + '参数初始化')
      var item = getItemByRef(param.ref, state.json)
      Vue.set(item, 'options', param.options)
    },
    // 属性修改：更新options
    propUpdateItemOptions(state, param) {
      const item = getItemByRef(param.ref, state.json)
      // 避免不同组件的itemClick，也会调用该方法，从而多余的更新
      if (JSON.stringify(item.options) != JSON.stringify(param.obj)) {
        Vue.set(item, 'options', deepClone(param.obj))
        console.log('属性修改：store -> ' + param.ref + '更新组件options')
        // 添加历史记录
        this.commit('historyAdd', {
          type: 'propUpdateItemOptions',
          msg: '组件属性更新【' + param.ref + '】'
        })
      }
    },

    // 拖拽：添加组件
    mainAddItem(state, param) {
      console.log('拖拽：store -> ' + param.parentRef + '添加组件')
      this.commit('addItem', param)

      // 添加历史记录
      this.commit('historyAdd', {
        type: 'mainAddItem',
        msg: '添加组件【' + param.obj.type + '】'
      })
    },
    // 拖拽：同一容器交换位置
    mainSwapItemWithSameCtn(state, param) {
      console.log('拖拽：store -> ' + param.ref + '组件交换位置')
      const parent = getParentItemByRef(param.ref, state.json)
      const children = parent.children
      const newItem = deepClone(children[param.newIndex])
      const oldItem = deepClone(children[param.oldIndex])
      Vue.set(children, param.newIndex, oldItem)
      Vue.set(children, param.oldIndex, newItem)
      // 添加历史记录
      this.commit('historyAdd', {
        type: 'mainSwapItemWithSameCtn',
        msg: '容器交换位置【' + param.ref + '】'
      })
    },
    // 拖拽：不同容器交换位置
    mainSwapItemWithDiffCtn(state, param) {
      console.log('拖拽：store -> ' + param.ref + '组件交换位置')
      this.commit('removeItem', param)
      this.commit('addItem', param)
      // 添加历史记录
      this.commit('historyAdd', {
        type: 'mainSwapItemWithDiffCtn',
        msg: '容器位置修改【' + param.ref + '】'
      })
    },

    // 树结构拖动：先删除，再增加
    treeUpdate(state, param) {
      console.log('树：拖拽更新')
      const draggingItem = getItemByRef(param.draggingNodeRef, state.json)
      const dropNodeItem = getItemByRef(param.dropNodeRef, state.json)

      var parentItem = getParentItemByRef(param.dropNodeRef, state.json)
      var parentRef = ''
      var dataIndex = ''

      // 移除拖的组件
      this.commit('removeItem', {
        ref: param.draggingNodeRef
      })

      switch (param.dropType) {
        case 'before':
          parentRef = parentItem.ref
          dataIndex = getItemIndexByRef(param.dropNodeRef, state.json)
          break
        case 'after':
          parentRef = parentItem.ref
          dataIndex = getItemIndexByRef(param.dropNodeRef, state.json) + 1
          break
        case 'inner':
          parentRef = param.dropNodeRef
          dataIndex = dropNodeItem.children.length
          break
      }
      // 添加组件
      this.commit('addItem', {
        parentRef: parentRef,
        dataIndex: dataIndex,
        obj: deepClone(draggingItem)
      })
      // 添加历史记录
      this.commit('historyAdd', {
        type: 'treeUpdate',
        msg: '树拖动位置【' + param.draggingNodeRef + '】'
      })
    },

    // 工具栏操作：重复
    toolRepeatItem(state, param) {
      console.log('工具栏：store -> ' + param.ref + '组件重复')
      const ref = param.ref
      const parent = getParentItemByRef(ref, state.json)
      const newItem = deepClone(getItemByRef(ref, state.json))
      setNewRef(newItem)
      const children = parent.children
      for (let i = 0; i < children.length; i++) {
        if (children[i].ref == ref) {
          children.splice(i + 1, 0, newItem)
          // 添加历史记录
          this.commit('historyAdd', {
            type: 'toolRepeatItem',
            msg: '重复操作【' + param.ref + '】'
          })
          return newItem.ref
        }
      }
    },
    // 工具栏操作：复制
    toolCopyItem(state, param) {
      console.log('工具栏：store -> ' + param.ref + '组件复制')
      const copyItem = deepClone(getItemByRef(param.ref, state.json))
      Vue.set(state.tempData, 'copyItem', copyItem)
      // 添加历史记录
      this.commit('historyAdd', {
        type: 'toolCopyItem',
        msg: '复制操作【' + param.ref + '】'
      })
    },
    // 工具栏操作：粘贴
    toolPasteItem(state, param) {
      console.log('工具栏：store -> ' + param.ref + '组件粘贴')
      const ref = param.ref
      const parent = getParentItemByRef(ref, state.json)
      const newItem = deepClone(state.tempData.copyItem)
      setNewRef(newItem)
      const children = parent.children
      for (let i = 0; i < children.length; i++) {
        if (children[i].ref == ref) {
          children.splice(i + 1, 0, newItem)
          return newItem.ref
        }
      }
      // 添加历史记录
      this.commit('historyAdd', {
        type: 'toolPasteItem',
        msg: '粘贴操作【' + param.ref + '】'
      })
    },
    // 工具栏操作：删除
    toolRemoveItem(state, param) {
      console.log('工具栏：store -> ' + param.ref + '组件删除')
      this.commit('removeItem', param)

      // 重置当前选择
      this.commit('resetCurrentItem')

      // 添加历史记录
      this.commit('historyAdd', {
        type: 'toolRemoveItem',
        msg: '删除操作【' + param.ref + '】'
      })
    },
    // 工具栏操作：回滚
    toolRevertStore(state, param) {
      const index = param.index
      Vue.set(state, 'json', deepClone(state.history.data[index].data))
      // 重置当前选择
      this.commit('resetCurrentItem')
      console.log(state.json)
    },
    revokeOperate(state) {
      const hisData = state.history.data

      if (hisData.length - 1 <= state.history.currentIndex) {
        console.log('已经是最后一个操作了，不能回滚了')
        return
      }
      state.history.currentIndex++
      Vue.set(state, 'json', deepClone(hisData[state.history.currentIndex].data))
    },
    redoOperate(state) {
      const hisData = state.history.data

      if (state.history.currentIndex <= 0) {
        console.log('已经是最新操作了，不能重做了')
        return
      }
      state.history.currentIndex--
      Vue.set(state, 'json', deepClone(hisData[state.history.currentIndex].data))
    },
    // 更新当前设计的页面
    updateCurrentDesignPanel(state, param) {
      state.tempData.currentDesignPanel = param
    },
    // 更新当前设计的页面
    updateCurrentModel(state, param) {
      state.tempData.currentModel = param
    }


  }

})




export default apDevStore
